//
// edip_param.hpp
//
// LGPL Version 2.1 HEADER START
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
//
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301  USA
//
// LGPL Version 2.1 HEADER END
//

//
// Copyright (c) 2021, Regents of the University of Minnesota.
// All rights reserved.
//
// Contributors:
//    Yaser Afshar
//

#ifndef EDIP_PARAM_HPP
#define EDIP_PARAM_HPP

/*!
 * \brief EDIP parameter class
 *
 * Reference:
 *  Justo, Jo\~ao F. and Bazant, Martin Z. and Kaxiras, Efthimios and Bulatov,
 *  V. V. and Yip, Sidney, "Interatomic potential for silicon defects and
 *  disordered phases," Phys. Rev.B, 58(5):2539--2550, Aug 1998.
 */
struct EDIPParam {
  /*! A coefficient for pair interaction I-J (EQ. 4) */
  double A;
  /*! B coefficient for pair interaction I-J (EQ. 4) */
  double B;
  /*! Cut-off distance for pair interaction I-J (EQ. 3) */
  double cutoffA;
  /*! Lower cut-off distance for calculating Z_I (EQ. 3) */
  double cutoffC;
  /*! alpha coefficient for calculating Z_I, (EQ. 3) */
  double alpha;
  /*! beta coefficient, for calculating in Gaussian function (EQ. 5) */
  double beta;
  /*! sigma coefficient for pair interaction I-J (EQ. 4) */
  double sigma;
  /*! rho coefficient for pair interaction I-J (EQ. 4) */
  double rho;
  /*! gamma coefficient for three-body interaction I-J-K (EQ. 7) */
  double gamma;
  /*! eta coefficient for function h(l,Z) (EQ. 9) */
  double eta;
  /*! lambda coefficient for function h(l,Z) (EQ. 9) */
  double lambda;
  /*! mu coefficient for function Q(Z) (EQ. 9) */
  double mu;
  /*! Q0 coefficient for function Q(Z) (EQ. 9) */
  double Q0;
  /*! u1 coefficient for function tau(Z) (EQ. 10) */
  double u1;
  /*! u2 coefficient for function tau(Z) (EQ. 10) */
  double u2;
  /*! u3 coefficient for function tau(Z) (EQ. 10) */
  double u3;
  /*! u4 coefficient for function tau(Z) (EQ. 10) */
  double u4;
  /*! Cut-off distance square for pair interaction */
  double cutsq;
};

#endif  // EDIP_PARAM
